@@ -1,3 +1,17 @@
+# Copyright 2022 Cisco Systems, Inc. and/or its affiliates.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 from __future__ import annotations
 
 import asyncio
@@ -6,7 +20,7 @@ import os
 import random
 import shutil
 import signal
-from typing import Any, Optional
+from typing import Any, Optional, Union
 
 import backoff
 import colorama
@@ -26,7 +40,7 @@ from servo.servo import _set_current_servo
 from servo.types import Adjustment, Control, Description, Duration, Measurement
 
 
-class ServoRunner(pydantic.BaseModel, servo.logging.Mixin, servo.api.Mixin):
+class ServoRunner(pydantic.BaseModel, servo.logging.Mixin):
     interactive: bool = False
     _servo: servo.Servo = pydantic.PrivateAttr(None)
     _connected: bool = pydantic.PrivateAttr(False)
@@ -58,18 +72,15 @@ class ServoRunner(pydantic.BaseModel, servo.logging.Mixin, servo.api.Mixin):
         return self._connected
 
     @property
-    def optimizer(self) -> servo.Optimizer:
+    def optimizer(
+        self,
+    ) -> servo.configuration.OptimizerTypes:
         return self.servo.optimizer
 
     @property
     def config(self) -> servo.BaseServoConfiguration:
         return self.servo.config
 
-    @property
-    def api_client_options(self) -> dict[str, Any]:
-        # Adopt the servo config for driving the API mixin
-        return self.servo.api_client_options
-
     async def describe(self, control: Control) -> Description:
         self.logger.info("Describing...")
 
@@ -86,7 +97,7 @@ class ServoRunner(pydantic.BaseModel, servo.logging.Mixin, servo.api.Mixin):
 
     async def measure(self, param: servo.api.MeasureParams) -> Measurement:
         if isinstance(param, dict):
-            # required parsing has failed in api.Mixin._post_event(), run parse_obj to surface the validation errors
+            # required parsing has failed in servo.post_event(), run parse_obj to surface the validation errors
             servo.api.MeasureParams.parse_obj(param)
         servo.logger.info(f"Measuring... [metrics={', '.join(param.metrics)}]")
         servo.logger.trace(devtools.pformat(param))
@@ -123,7 +134,7 @@ class ServoRunner(pydantic.BaseModel, servo.logging.Mixin, servo.api.Mixin):
         return aggregate_description
 
     async def exec_command(self) -> servo.api.Status:
-        cmd_response = await self._post_event(servo.api.Events.whats_next, None)
+        cmd_response = await self.servo.post_event(servo.api.Events.whats_next, None)
         self.logger.info(f"What's Next? => {cmd_response.command}")
         self.logger.trace(devtools.pformat(cmd_response))
 
@@ -137,7 +148,7 @@ class ServoRunner(pydantic.BaseModel, servo.logging.Mixin, servo.api.Mixin):
             self.logger.debug(devtools.pformat(description))
 
             status = servo.api.Status.ok(descriptor=description.__opsani_repr__())
-            return await self._post_event(servo.api.Events.describe, status.dict())
+            return await self.servo.post_event(servo.api.Events.describe, status.dict())
 
         elif cmd_response.command == servo.api.Commands.measure:
             try:
@@ -153,7 +164,7 @@ class ServoRunner(pydantic.BaseModel, servo.logging.Mixin, servo.api.Mixin):
                 self.logger.error(f"Responding with {param}")
                 self.logger.opt(exception=error).debug("Measure failure details")
 
-            return await self._post_event(servo.api.Events.measure, param)
+            return await self.servo.post_event(servo.api.Events.measure, param)
 
         elif cmd_response.command == servo.api.Commands.adjust:
             adjustments = servo.api.descriptor_to_adjustments(
@@ -178,7 +189,7 @@ class ServoRunner(pydantic.BaseModel, servo.logging.Mixin, servo.api.Mixin):
                 self.logger.error(f"Responding with {status.dict()}")
                 self.logger.opt(exception=error).debug("Adjust failure details")
 
-            return await self._post_event(servo.api.Events.adjust, status.dict())
+            return await self.servo.post_event(servo.api.Events.adjust, status.dict())
 
         elif cmd_response.command == servo.api.Commands.sleep:
             # TODO: Model this
@@ -279,7 +290,7 @@ class ServoRunner(pydantic.BaseModel, servo.logging.Mixin, servo.api.Mixin):
             f"Servo started with {len(self.servo.connectors)} active connectors [{self.optimizer.id} @ {self.optimizer.url or self.optimizer.base_url}]"
         )
 
-        async def giveup() -> None:
+        async def giveup(_: dict) -> None:
             loop = asyncio.get_event_loop()
             self.logger.critical("retries exhausted, giving up")
             asyncio.create_task(self.shutdown(loop))
@@ -295,7 +306,7 @@ class ServoRunner(pydantic.BaseModel, servo.logging.Mixin, servo.api.Mixin):
             )
             async def connect() -> None:
                 self.logger.info("Saying HELLO.", end=" ")
-                await self._post_event(
+                await self.servo.post_event(
                     servo.api.Events.hello,
                     dict(
                         agent=servo.api.user_agent(),
@@ -332,7 +343,9 @@ class ServoRunner(pydantic.BaseModel, servo.logging.Mixin, servo.api.Mixin):
         try:
             self._running = False
             if self.connected:
-                await self._post_event(servo.api.Events.goodbye, dict(reason=reason))
+                await self.servo.post_event(
+                    servo.api.Events.goodbye, dict(reason=reason)
+                )
         except Exception:
             self.logger.exception(f"Exception occurred during GOODBYE request")
 
@@ -419,7 +432,7 @@ class AssemblyRunner(pydantic.BaseModel, servo.logging.Mixin):
                     status = servo.api.Status.from_error(error)
                     self.logger.error(f"Responding with {status.dict()}")
                     runner = self._runner_for_servo(servo.current_servo())
-                    await runner._post_event(operation, status.dict())
+                    await runner.servo.post_event(operation, status.dict())
 
                 tasks = [
                     t for t in asyncio.all_tasks() if t is not asyncio.current_task()
