@@ -1,3 +1,17 @@
+# Copyright 2022 Cisco Systems, Inc. and/or its affiliates.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 """The `servo.types` module defines the essential data types shared by all
 consumers of the servo package.
 """
@@ -58,6 +72,7 @@ def _orjson_dumps(
 
             return default(obj)
         except TypeError:
+            # TODO increase visibility into this error as this fallback can cause issues with round tripping (eg. AnyHttpUrl)
             return orjson.dumps(obj).decode()
 
     try:
@@ -68,6 +83,7 @@ def _orjson_dumps(
 
 DEFAULT_JSON_ENCODERS = {
     pydantic.SecretStr: lambda v: v.get_secret_value() if v else None,
+    pydantic.AnyHttpUrl: str,
 }
 
 
